@@ -109,20 +109,36 @@ lvalue_or_else (ref, string)
 /* INIT is a CALL_EXPR which needs info about its target.
    TYPE is the type that this initialization should appear to have.
 
-   Build an encapsultation of the initialization to perfom
+   Build an encapsulation of the initialization to perform
    and return it so that it can be processed by language-independent
-   and language-specific expression expanders.  */
+   and language-specific expression expanders.
+
+   If WITH_CLEANUP_P is nonzero, we build a cleanup for this expression.
+   Otherwise, cleanups are not built here.  For example, when building
+   an initialization for a stack slot, since the called function handles
+   the cleanup, we would not want to do it here.  */
 tree
-build_cplus_new (type, init)
+build_cplus_new (type, init, with_cleanup_p)
      tree type;
      tree init;
+     int with_cleanup_p;
 {
   tree slot = build (VAR_DECL, type);
   tree rval = build (CPLUS_NEW_EXPR, type,
 		     TREE_OPERAND (init, 0), TREE_OPERAND (init, 1), slot);
   TREE_ADDRESSABLE (rval) = 1;
   rval = build (NEW_EXPR, type, slot, rval, 0);
+  TREE_VOLATILE (rval) = 1;
   TREE_ADDRESSABLE (rval) = 1;
+
+  if (with_cleanup_p && TYPE_NEEDS_DESTRUCTOR (type))
+    rval = build (WITH_CLEANUP_EXPR, type, rval, 0,
+		  build_delete (TYPE_POINTER_TO (type),
+				build_unary_op (ADDR_EXPR, slot, 0),
+				integer_two_node,
+				LOOKUP_NORMAL|LOOKUP_DESTRUCTOR|LOOKUP_NONVIRTUAL, 0));
+
+  TREE_VOLATILE (rval) = 1;
   return rval;
 }
 
