@@ -60,12 +60,16 @@ struct lang_id2
 
 /* Macros for access to language-specific slots in an identifier.  */
 
-#define IDENTIFIER_GLOBAL_VALUE(NODE)	\
+#define IDENTIFIER_GLOBAL_VALUE(NODE)		\
   (((struct lang_identifier *)(NODE))->global_value)
-#define IDENTIFIER_CLASS_VALUE(NODE)	\
+#define IDENTIFIER_CLASS_VALUE(NODE)		\
   (((struct lang_identifier *)(NODE))->class_value)
-#define IDENTIFIER_LOCAL_VALUE(NODE)	\
+#define IDENTIFIER_LOCAL_VALUE(NODE)		\
   (((struct lang_identifier *)(NODE))->local_value)
+#define IDENTIFIER_AS_LIST(NODE)		\
+  ((tree)((struct lang_identifier *)(NODE))->x)
+#define SET_IDENTIFIER_AS_LIST(NODE,LIST)	\
+  (((struct lang_identifier *)(NODE))->x = (struct lang_id2*)(LIST))
 
 #define IDENTIFIER_LABEL_VALUE(NODE)	\
   (((struct lang_identifier *)(NODE))->x \
@@ -92,8 +96,9 @@ extern int pedantic;
 /* In a RECORD_TYPE or UNION_TYPE, nonzero if any component is read-only.  */
 #define C_TYPE_FIELDS_READONLY(type) TYPE_SEP_UNIT (type)
 
-/* in c-typecheck.c */
-extern tree build_component_ref(), build_conditional_expr(), build_compound_expr();
+/* in c-typeck.c */
+extern tree build_component_ref(), build_conditional_expr();
+extern tree build_x_compound_expr (), build_compound_expr();
 extern tree build_unary_op(), build_binary_op(), build_function_call();
 extern tree build_binary_op_nodefault ();
 extern tree build_indirect_ref(), build_array_ref(), build_c_cast();
@@ -160,6 +165,11 @@ extern tree ridpointers[];
 
 extern int dollars_in_ident;
 
+/* Nonzero causes a call to __main() to be emmitted as first instruction of */
+/* main() */
+
+extern int flag__main;
+
 /* Nonzero means allow type mismatches in conditional expressions;
    just make their values `void'.   */
 
@@ -212,6 +222,9 @@ extern int flag_traditional;
 
 extern int write_virtuals;
 
+/* Nonzero means that we are in an "interface" section of the compiler.  */
+extern int interface_only, interface_unknown;
+
 /* Nonzero means we should attempt to elide constructors when possible.  */
 
 extern int flag_elide_constructors;
@@ -233,6 +246,10 @@ extern int flag_default_inline;
 /* Nonzero means recognize and handle exception handling constructs.  */
 
 extern int flag_no_inline;
+
+/* Nonzero means emit cadillac protocol.  */
+
+extern int flag_cadillac;
 
 /* C++ language-specific tree codes.  */
 #define DEFTREECODE(SYM, NAME, TYPE, LENGTH) SYM,
@@ -254,6 +271,8 @@ enum languages { lang_c, lang_cplusplus };
 #define FNADDR_FROM_VTABLE_ENTRY(ENTRY) \
   TREE_VALUE (TREE_CHAIN (TREE_CHAIN (CONSTRUCTOR_ELTS (ENTRY))))
 
+enum conversion_type { ptr_conv, constptr_conv, int_conv, real_conv, last_conversion_type };
+
 /* Statistics show that while the GNU C++ compiler may generate
    thousands of different types during a compilation run, it
    generates relatively few (tens) of classtypes.  Because of this,
@@ -313,7 +332,9 @@ struct lang_type
       unsigned has_default_ctor : 1;
       unsigned gets_const_assign_ref : 1;
       unsigned has_const_assign_ref : 1;
-      unsigned dummy4 : 4;
+      unsigned interface_only : 1;
+      unsigned interface_unknown : 1;
+      unsigned dummy3 : 2;
     } type_flags;
 
   unsigned n_parents : 16;
@@ -350,13 +371,13 @@ struct lang_type
   union tree_node *base_init_list;
   union tree_node *abstract_virtuals;
   union tree_node *as_list;
+  union tree_node *as_id_list;
   union tree_node *vtbl_ptr;
   union tree_node *instance_variable;
   union tree_node *friend_classes;
 
   char *mi_matrix;
-  union tree_node *ptr_conversion;
-  union tree_node *constptr_conversion;
+  union tree_node *conversions[last_conversion_type];
   union tree_node *wrap_type;
 
 #ifdef SOS
@@ -388,20 +409,20 @@ struct lang_type
    base class is via a `public' declaration, which preserves public
    fields from the base class as public.
    OVERLOADED.  */
-#define TREE_VIA_PUBLIC(NODE) ((NODE)->common.external_attr)
+#define TREE_VIA_PUBLIC(NODE) ((NODE)->common.used_attr) /* overloaded! */
 
 /* Nonzero for TREE_LIST node means that the path to the
    base class is via a `protected' declaration, which preserves
    protected fields from the base class as protected.
    OVERLOADED.  */
-#define TREE_VIA_PROTECTED(NODE) ((NODE)->common.public_attr)
+#define TREE_VIA_PROTECTED(NODE) ((NODE)->common.literal_attr) /* overloaded! */
 
 /* Nonzero for a _CLASSTYPE node which we know to be private.  */
 #define TYPE_PRIVATE_P(NODE) (TYPE_LANG_SPECIFIC(NODE)->type_flags.private_attr)
 
 /* Nonzero for a _CLASSTYPE node means that the derivation chain is via
-   a `virtual' declaration. OVERLOADED.  */
-#define TREE_VIA_VIRTUAL(NODE) ((NODE)->common.public_attr)
+   a `virtual' declaration.  */
+#define TREE_VIA_VIRTUAL(NODE) ((NODE)->common.literal_attr) /* overloaded! */
 
 /* Nonzero means that this _CLASSTYPE node defines ways of converting
    itself to other types.  */
@@ -484,7 +505,7 @@ struct lang_type
 /* When building a matrix to determine by a single lookup
    whether one class is derived from another or not,
    this field is the index of the class in the table.  */
-#define CLASSTYPE_CID(NODE) (TYPE_LANG_SPECIFIC(NODE)->cid)
+#define CLASSTYPE_CID(NODE) (TYPE_LANG_SPECIFIC(TYPE_MAIN_VARIANT (NODE))->cid)
 
 #define CLASSTYPE_TAGS(NODE) (TYPE_LANG_SPECIFIC(NODE)->tags)
 #define CLASSTYPE_NAMES(NODE) (TYPE_LANG_SPECIFIC(NODE)->names)
@@ -603,6 +624,8 @@ struct lang_type
    is the main variant of the type, and whose TREE_VIA_PUBLIC
    and TREE_VIA_VIRTUAL bits are correctly set.  */
 #define CLASSTYPE_AS_LIST(NODE) (TYPE_LANG_SPECIFIC(NODE)->as_list)
+/* Same, but cache a list whose value is the name of this type.  */
+#define CLASSTYPE_AS_ID_LIST(NODE) (TYPE_LANG_SPECIFIC(NODE)->as_id_list)
 
 /* Slot in which to cache a copy of the local vtable pointer.  */
 #define CLASSTYPE_VTBL_PTR(NODE) (TYPE_LANG_SPECIFIC(NODE)->vtbl_ptr)
@@ -620,8 +643,7 @@ struct lang_type
 /* If there is exactly one conversion to a non-void, non-const pointer type,
    remember that here.  If there are more than one, put
    `error_mark_node' here.  If there are none, this holds NULL_TREE.  */
-#define CLASSTYPE_PTR_CONVERSION(NODE) (TYPE_LANG_SPECIFIC(NODE)->ptr_conversion)
-#define CLASSTYPE_CONSTPTR_CONVERSION(NODE) (TYPE_LANG_SPECIFIC(NODE)->constptr_conversion)
+#define CLASSTYPE_CONVERSION(NODE,KIND) (TYPE_LANG_SPECIFIC(NODE)->conversions[KIND])
 
 /* Nonzero means that class is "dynamic" in SOS sense.  (IRIA-specific.)  */
 #define TYPE_DYNAMIC(NODE) (TYPE_LANG_SPECIFIC(NODE)->type_flags.dynamic)
@@ -646,6 +668,13 @@ struct lang_type
 /* Nonzero if this class has ref members which have no specified initialization.  */
 #define CLASSTYPE_REF_FIELDS_NEED_INIT(NODE) (TYPE_LANG_SPECIFIC(NODE)->type_flags.ref_needs_init)
 
+/* Nonzero if this class is included from a header file which employs
+   `#pragma interface', and it is not included in its implementation file.  */
+#define CLASSTYPE_INTERFACE_ONLY(NODE) (TYPE_LANG_SPECIFIC(NODE)->type_flags.interface_only)
+
+/* Same as above, but for classes whose purpose we do not know.  */
+#define CLASSTYPE_INTERFACE_UNKNOWN(NODE) (TYPE_LANG_SPECIFIC(NODE)->type_flags.interface_unknown)
+
 /* Depending on a class's "owner" they may have different properties,
    such as the offset which must be added to `this', and the
    virtual function table with which they are initialized.  This is
@@ -673,12 +702,12 @@ struct lang_type
 
 /* Nonzero for TREE_LIST node means that this list of things
    is a list of parameters, as opposed to a list of expressions.  */
-#define TREE_PARMLIST(NODE) ((NODE)->common.regdecl_attr) /* overloaded! */
+#define TREE_PARMLIST(NODE) TREE_UNSIGNED (NODE) /* overloaded! */
 
 /* Nonzero for FIELD_DECL node means that this FIELD_DECL is
    a member of an anonymous union construct.  The name of the
    union is .  */
-#define TREE_ANON_UNION_ELEM(NODE) ((NODE)->common.regdecl_attr) /* overloaded! */
+#define TREE_ANON_UNION_ELEM(NODE) TREE_REGDECL (NODE) /* overloaded! */
 
 /* For FUNCTION_TYPE or METHOD_TYPE, a list of the exceptions that
    this type can raise.  */
@@ -727,6 +756,10 @@ struct lang_decl
 #endif
 };
 
+/* Non-zero if NODE is a _DECL with TREE_READONLY set.  */
+#define TREE_READONLY_DECL_P(NODE) \
+  (TREE_READONLY (NODE) && *(tree_code_type[TREE_CODE (NODE)]) == 'd')
+
 /* For FUNCTION_DECLs: return the language in which this decl
    was declared.  */
 #define DECL_LANGUAGE(NODE) (DECL_LANG_SPECIFIC(NODE)->decl_flags.language)
@@ -788,7 +821,7 @@ struct lang_decl
 
 /* Nonzero for FIELD_DECL node means that this FIELD_DECL is
    a member of an anonymous union construct.  */
-#define DECL_ANON_UNION_ELEM(NODE) ((NODE)->common.regdecl_attr) /* overloaded! */
+#define DECL_ANON_UNION_ELEM(NODE) TREE_REGDECL (NODE) /* overloaded! */
 
 /* For a FUNCTION_DECL: the name of the function before being overloaded.  */
 #define DECL_ORIGINAL_NAME(NODE) (DECL_LANG_SPECIFIC(NODE)->original_name)
@@ -828,6 +861,10 @@ struct lang_decl
    has been declared virtual.  */
 #define DECL_VIRTUAL_P(NODE) (TREE_LANG_FLAG_2 (NODE))
 
+/* Nonzero for FIELD_DECL means that this field is a
+   virtual baseclass field.  Used for printing debugging info.  */
+#define DECL_VBASE_P(NODE) ((NODE)->common.external_attr)
+
 /* Nonzero for FIELD_DECLs means that this field is private,
    and can only be accessed within the scope of the class
    which defines it (or its friends).  */
@@ -860,7 +897,10 @@ struct lang_decl
    is a call to a constructor.
 
    When appearing in a CONSTRUCTOR, it means that it was
-   a GNU C constructor expression.  */
+   a GNU C constructor expression.
+
+   When appearing in a FIELD_DECL, it means that this field
+   has been duly initialized in its constructor.  */
 #define TREE_HAS_CONSTRUCTOR(NODE) (TREE_LANG_FLAG_1(NODE))
 
 /* Nonzero for _TYPE means that the _TYPE defines a destructor.  */
@@ -947,7 +987,7 @@ struct lang_decl
 #define DECL_VCONTEXT(NODE) (DECL_LANG_SPECIFIC(NODE)->vcontext)
 
 /* DECL_FCONTEXT is the *first* baseclass in which this FIELD_DECL
-   is defined.  */
+   is defined.  This only applies to vfield and vbase decls.  */
 #define SET_DECL_FCONTEXT(NODE,VALUE) (DECL_LANG_SPECIFIC(NODE) = (struct lang_decl *)(VALUE))
 #define DECL_FCONTEXT(NODE) ((tree)DECL_LANG_SPECIFIC(NODE))
 
@@ -956,7 +996,8 @@ struct lang_decl
    is where we stash the information.  */
 #define DECL_STATIC_CONTEXT(NODE) (DECL_LANG_SPECIFIC(NODE)->vcontext)
 
-/* C++: all of these are overloaded!  These apply to PARM_DECLs and VAR_DECLs.  */
+/* C++: all of these are overloaded!
+   These apply to PARM_DECLs and VAR_DECLs and RESULT_DECLs.  */
 #define DECL_REFERENCE_SLOT(NODE) ((tree)(NODE)->decl.arguments)
 #define SET_DECL_REFERENCE_SLOT(NODE,VAL) ((NODE)->decl.arguments=VAL)
 
@@ -1032,6 +1073,8 @@ extern tree grokopexpr (), getaggrs (), groktypefield ();
 extern tree grok_method_quals (), grok_enum_decls ();
 extern void finish_anon_union();
 extern tree long_long_integer_type_node, long_long_unsigned_type_node;
+/* For building calls to `delete'.  */
+extern tree integer_two_node, integer_three_node;
 extern tree get_first_matching_virtual (), get_abstract_virtuals ();
 
 /* in cplus-typeck.c */
@@ -1049,7 +1092,7 @@ extern tree build_x_arrow ();
 extern tree build_component_ref_1 ();
 extern tree datatype (), unary_complex_lvalue (), target_type ();
 extern tree build_return_stmt ();
-extern tree actualparameterlist ();
+extern tree actualparameterlist (), commonparms ();
 extern tree cplus_size_in_bytes ();
 extern tree cplus_sizeof (), cplus_sizeof_nowarn ();
 extern tree error_mark_list;
@@ -1079,6 +1122,7 @@ extern tree current_exception_object;
 extern tree build_exception_variant ();
 extern tree lookup_exception_type (), lookup_exception_cname ();
 extern tree lookup_exception_object ();
+extern tree cplus_expand_start_catch ();
 extern tree cplus_expand_end_try ();
 
 /* in cplus-class.c */
@@ -1088,8 +1132,10 @@ extern tree prev_class_type;
 
 extern tree current_lang_name, lang_name_cplusplus, lang_name_c;
 
-extern tree hack_identifier (), hack_operator (), hack_wrapper ();
-extern tree convert_pointer_to (), convert_to_reference (), convert_to_aggr (), convert_aggr ();
+extern tree do_identifier (), hack_identifier ();
+extern tree hack_operator (), hack_wrapper ();
+extern tree convert_pointer_to (), convert_pointer_to_vbase ();
+extern tree convert_to_reference (), convert_to_aggr (), convert_aggr ();
 extern tree build_x_new (), build_x_delete ();
 extern tree build_new (), build_vec_new (), build_delete (), build_vec_delete ();
 extern tree make_destructor_name ();
@@ -1151,14 +1197,14 @@ extern int current_function_parms_stored;
 
 #define JOINER '$'
 
-#define THIS_NAME "$this"
-#define VPTR_NAME "$vptr"
+#define THIS_NAME "$t"
+#define VPTR_NAME "$v"
 #define THROW_NAME "$eh_throw"
 #define DESTRUCTOR_DECL_FORMAT "_$_%s"
 #define WRAPPER_DECL_FORMAT "__W$%s"
 #define WRAPPER_PRED_DECL_FORMAT "__P$%s"
 #define ANTI_WRAPPER_DECL_FORMAT "__w$%s"
-#define IN_CHARGE_NAME "__in$charge__"
+#define IN_CHARGE_NAME "__in$chrg"
 #define AUTO_VTABLE_NAME "__vtbl$me__"
 #define AUTO_TEMP_NAME "_$tmp_"
 #define AUTO_TEMP_FORMAT "_$tmp_%d"
@@ -1183,14 +1229,14 @@ extern int current_function_parms_stored;
 
 #define JOINER '.'
 
-#define THIS_NAME ".this"
-#define VPTR_NAME ".vptr"
+#define THIS_NAME ".t"
+#define VPTR_NAME ".v"
 #define THROW_NAME ".eh_throw"
 #define DESTRUCTOR_DECL_FORMAT "_._%s"
 #define WRAPPER_DECL_FORMAT "__W.%s"
 #define WRAPPER_PRED_DECL_FORMAT "__P.%s"
 #define ANTI_WRAPPER_DECL_FORMAT "__w.%s"
-#define IN_CHARGE_NAME "__in.charge__"
+#define IN_CHARGE_NAME "__in.chrg"
 #define AUTO_VTABLE_NAME "__vtbl.me__"
 #define AUTO_TEMP_NAME "_.tmp_"
 #define AUTO_TEMP_FORMAT "_.tmp_%d"
@@ -1222,11 +1268,13 @@ extern int current_function_parms_stored;
 #define OPERATOR_METHOD_LENGTH 13
 #define OPERATOR_NEW_LENGTH 5
 #define OPERATOR_DELETE_LENGTH 8
+#define EXCEPTION_NAME_LENGTH 12
 #define FILE_FUNCTION_PREFIX_LEN 9
 #define VTABLE_DELTA_NAME "delta"
 #define VTABLE_DELTA2_NAME "delta2"
 #define VTABLE_INDEX_NAME "index"
 #define VTABLE_PFN_NAME "pfn"
+#define EXCEPTION_CLEANUP_NAME "exception cleanup"
 
 #define THIS_NAME_P(ID_NODE) (IDENTIFIER_POINTER (ID_NODE)[0] == JOINER \
 			      && IDENTIFIER_POINTER (ID_NODE)[1] == 't')
@@ -1452,7 +1500,8 @@ struct candidate *ideal_candidate ();
      to be non-visibile to current scope, call it anyway.
    LOOKUP_DYNAMIC means call dynamic functions, a la SOS.
    LOOKUP_NO_CONVERSION means that user-defined conversions are not
-     permitted.  Built-in conversions are permitted.  */
+     permitted.  Built-in conversions are permitted.
+   LOOKUP_DESTRUCTOR means explicit call to destructor.  */
 
 #define LOOKUP_PROTECT (1)
 #define LOOKUP_COMPLAIN (2)
@@ -1465,6 +1514,7 @@ struct candidate *ideal_candidate ();
 #define LOOKUP_PROTECTED_OK (128)
 #define LOOKUP_DYNAMIC (256)
 #define LOOKUP_NO_CONVERSION (512)
+#define LOOKUP_DESTRUCTOR (512)
 
 /* Anatomy of a DECL_FRIENDLIST (which is a TREE_LIST):
    purpose = friend name (IDENTIFIER_NODE);
@@ -1512,7 +1562,7 @@ extern tree get_vbase_types ();
 extern tree get_baselinks ();
 extern tree get_wrapper ();
 extern tree make_assoc (), copy_assoc ();
-extern tree assoc_value ();
+extern tree assoc_value (), virtual_member ();
 
 #define PRINT_LANG_DECL
 #define PRINT_LANG_TYPE
