@@ -1106,7 +1106,7 @@ build_functional_cast (exp, parms)
       return error_mark_node;
     }
 
-  if (! TYPE_NEEDS_CONSTRUCTOR (type))
+  if (! TYPE_NEEDS_CONSTRUCTOR (type) && parms != NULL_TREE)
     {
       if (TREE_CHAIN (parms) == NULL_TREE)
 	{
@@ -1135,6 +1135,7 @@ build_functional_cast (exp, parms)
 	    break;
 	  /* Hack for MI.  */
 	  i = CLASSTYPE_N_BASECLASSES (type);
+	  if (i == 0) break;
 	  while (i > 0)
 	    {
 	      if (TYPE_HAS_CONSTRUCTOR (CLASSTYPE_BASECLASS (type, i)))
@@ -1172,7 +1173,7 @@ build_functional_cast (exp, parms)
 
     if (rval != error_mark_node)
       {
-	rval = build_cplus_new (type, rval);
+	rval = build_cplus_new (type, rval, 1);
 	return rval;
       }
 
