@@ -24,14 +24,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    Error messages and low-level interface to malloc also handled here.  */
 
 #include "config.h"
+#include <sys/types.h>
 #include <stdio.h>
 #include <signal.h>
 #include <setjmp.h>
 
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
 
+/* Define getpagesize () if the system does not.  */
+#include "getpagesize.h"
+
 #ifdef NO_UNISTD_H
 #define SEEK_SET L_SET
 #define SEEK_CUR L_INCR
@@ -62,8 +65,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "rtl.h"
 #include "flags.h"
 
-#ifndef _bufsiz
-#define _bufsiz(p) ((p)->_bufsiz)
+#ifndef Z_BUFSIZ
+#define Z_BUFSIZ(p) ((p)->_bufsiz)
 #endif
 
 /* Change by Bryan Boreham, Kewill, Sun Jul 23 14:41:26 1989.
@@ -213,11 +216,6 @@ int flag_float_store = 0;
 
 int flag_combine_regs = 0;
 
-/* Nonzero for -fcse-follow-jumps:
-   have cse follow jumps to do a more extensive job.  */
-
-int flag_cse_follow_jumps;
-
 /* Nonzero enables strength-reduction in loop.c.  */
 
 int flag_strength_reduce = 0;
@@ -250,8 +248,9 @@ int flag_volatile;
 
 int flag_syntax_only = 0;
 
-/* Nonzero means do stupid register allocation.
-   Currently, this is 1 if `optimize' is 0.  */
+/* Nonzero means do stupid register allocation.  -noreg.
+   This and `optimize' are controlled by different switches in cc1,
+   but normally cc controls them both with the -O switch.  */
 
 int obey_regdecls = 0;
 
@@ -375,13 +374,12 @@ int g_switch_used = 0;
     if `-fSTRING' is seen as an option.
    (If `-fno-STRING' is seen as an option, the opposite value is stored.)  */
 
-static struct { char *string; int *variable; int on_value;} f_options[] =
+struct { char *string; int *variable; int on_value;} f_options[] =
 {
   {"float-store", &flag_float_store, 1},
   {"volatile", &flag_volatile, 1},
   {"defer-pop", &flag_defer_pop, 1},
   {"omit-frame-pointer", &flag_omit_frame_pointer, 1},
-  {"cse-follow-jumps", &flag_cse_follow_jumps, 1},
   {"strength-reduce", &flag_strength_reduce, 1},
   {"writable-strings", &flag_writable_strings, 1},
   {"peephole", &flag_no_peephole, 0},
@@ -427,7 +425,6 @@ int combine_time;
 int local_alloc_time;
 int global_alloc_time;
 int dbr_sched_time;
-int shorten_branch_time;
 int final_time;
 int symout_time;
 int dump_time;
@@ -706,6 +703,7 @@ error_for_asm (insn, s, v, v2)
      int v;			/* @@also used as pointer */
      int v2;			/* @@also used as pointer */
 {
+  rtx temp;
   char *filename;
   int line;
   rtx body = PATTERN (insn);
@@ -798,6 +796,41 @@ warning_with_decl (decl, s, v)
   fprintf (stderr, "\n");
 }
 
+/* Report a warning at the line number of the insn INSN.
+   S and V are a string and an arg for `printf'.
+   This is used only when INSN is an `asm' with operands,
+   and each ASM_OPERANDS records its own source file and line.  */
+
+void
+warning_for_asm (insn, s, v, v2)
+     rtx insn;
+     char *s;
+     int v;			/* @@also used as pointer */
+     int v2;			/* @@also used as pointer */
+{
+  char *filename;
+  int line;
+  rtx body = PATTERN (insn);
+  rtx asmop;
+
+  /* Find the (or one of the) ASM_OPERANDS in the insn.  */
+  if (GET_CODE (body) == SET && GET_CODE (SET_SRC (body)) == ASM_OPERANDS)
+    asmop = SET_SRC (body);
+  else if (GET_CODE (body) == ASM_OPERANDS)
+    asmop = body;
+  else if (GET_CODE (body) == PARALLEL
+	   && GET_CODE (XVECEXP (body, 0, 0)) == SET)
+    asmop = SET_SRC (XVECEXP (body, 0, 0));
+  else if (GET_CODE (body) == PARALLEL
+	   && GET_CODE (XVECEXP (body, 0, 0)) == ASM_OPERANDS)
+    asmop = XVECEXP (body, 0, 0);
+
+  filename = ASM_OPERANDS_SOURCE_FILE (asmop);
+  line = ASM_OPERANDS_SOURCE_LINE (asmop);
+
+  warning_with_file_and_line (filename, line, s, v, v2);
+}
+
 /* Apologize for not implementing some feature.
    S, V, and V2 are a string and args for `printf'.  */
 
@@ -971,7 +1004,6 @@ compile_file (name)
   local_alloc_time = 0;
   global_alloc_time = 0;
   dbr_sched_time = 0;
-  shorten_branch_time = 0;
   final_time = 0;
   symout_time = 0;
   dump_time = 0;
@@ -1192,7 +1224,7 @@ compile_file (name)
 	pfatal_with_name (previous_asm_file_name);
 #ifndef VMS
 #ifndef MASSCOMP
-      if (_bufsiz(asm_out_file) != _bufsiz(prev_file))
+      if (Z_BUFSIZ(asm_out_file) != Z_BUFSIZ(prev_file))
 #endif
 #endif
 	{
@@ -1208,22 +1240,22 @@ compile_file (name)
 
 	  fwrite (&dummy, 1, 1, asm_out_file);
 	  asm_out_file->_ptr = asm_out_file->_base;
-	  asm_out_file->_cnt = _bufsiz(asm_out_file);
+	  asm_out_file->_cnt = Z_BUFSIZ(asm_out_file);
 	  do
 	    {
-	      size = fread (asm_out_file->_base, 1, _bufsiz(asm_out_file), prev_file);
+	      size = fread (asm_out_file->_base, 1, Z_BUFSIZ(asm_out_file), prev_file);
 	      asm_out_file->_ptr += size;
 	      asm_out_file->_cnt -= size;
 	      fflush (asm_out_file);
 	    }
-	  while (size == _bufsiz(asm_out_file));
+	  while (size == Z_BUFSIZ(asm_out_file));
 	}
 #endif
 #endif
       fclose (prev_file);
     }
 
-#ifdef 0  /* This mode is discontinued.  */
+#if 0  /* This mode is discontinued.  */
   /* If GDB symbol table desired, open the GDB symbol output file.  */
   if (write_symbols == GDB_DEBUG)
     {
@@ -1384,8 +1416,16 @@ compile_file (name)
   if (write_symbols == SDB_DEBUG)
     TIMEVAR (symout_time,
 	     {
+	       tree decl;
 	       sdbout_tags (gettags ());
 	       sdbout_types (get_permanent_types ());
+	       /* Output first static file-scope vars, then global ones.  */
+	       for (decl = globals; decl; decl = TREE_CHAIN (decl))
+		 if (TREE_CODE (decl) == VAR_DECL && !TREE_PUBLIC (decl))
+		   sdbout_symbol (decl, 1);
+	       for (decl = globals; decl; decl = TREE_CHAIN (decl))
+		 if (TREE_CODE (decl) == VAR_DECL && TREE_PUBLIC (decl))
+		   sdbout_symbol (decl, 1);
 	     });
 #endif
 
@@ -1469,7 +1509,6 @@ compile_file (name)
       print_time ("combine", combine_time);
       print_time ("local-alloc", local_alloc_time);
       print_time ("global-alloc", global_alloc_time);
-      print_time ("shorten-branch", shorten_branch_time);
       print_time ("dbranch", dbr_sched_time);
       print_time ("final", final_time);
       print_time ("varconst", varconst_time);
@@ -1878,15 +1917,6 @@ rest_of_compilation (decl)
     }
 #endif
 
-#if 0
-  if (optimize)
-    /* Shorten branches.  */
-    TIMEVAR (shorten_branch_time,
-	     {
-	       shorten_branches (get_insns ());
-	     });
-#endif
-
   /* Now turn the rtl into assembler code.  */
 
   TIMEVAR (final_time,
@@ -1996,8 +2026,9 @@ void undump_data (path, name)
   int file;
   long file_size;
   extern int my_edata;
-  char buf[256] = "/usr/local/lib/";
+  char buf[256];
 
+  strcpy (buf, "/usr/local/lib/");
   strcat (buf, name);
 
   if ((file = open(name, O_RDONLY)) == -1)
@@ -2247,7 +2278,7 @@ main (argc, argv, envp)
 	      profile_block_flag = 1;
 #endif
 	    }
-#ifdef 0
+#if 0
 	  else if (!strcmp (str, "gg"))
 	    write_symbols = GDB_DEBUG;
 #endif
